% Rotate avw and write Analyze header file (*.hdr) and Analyze image files (*.img)
%
% function newavw=avw_rotate(avw,options)
% 
% Options are:
% Axis          - rotation along this axis: 'X' 'Y' 'Z'
%                 Default axis is Z
% center        - rotation center. Default center the mid points of the three
%                 dimension
% rfactor       - resampling factor: <1 produce bigger image
%                 rfcactor is applied just to axis that are perpendicular
%                 to Axis
% thetaZ        - rotation magnitude: rads
% fill_pixel    - pixel value use to fill out the "blank" part (corners) or
%                 borders after the resampling (the resampling
%                 interpolation produces some noise at the borders)
%
function newavw=avw_rotate(avw,options)
if ~isfield(options,'Resampling_factor')
    factors = [1,1,1];
    options.Resampling_factor =1;
end
if ~isfield(options,'Axis')
    direction = [1,2,3];
    dime = [2,3,4];
    factors = [options.Resampling_factor, options.Resampling_factor, 1];
else
    options.Axis=upper(options.Axis);
    if strcmp(upper(options.Axis), 'Z')
        direction = [1,2,3];
        dime = [2,3,4];
        factors = [options.Resampling_factor, options.Resampling_factor, 1];
    elseif strcmp(upper(options.Axis), 'Y')
        direction = [3,1,2];
        dime = [4,2,3];
        factors = [1,options.Resampling_factor, options.Resampling_factor];
    elseif strcmp(upper(options.Axis), 'X')
        direction = [3,1,2];
        dime = [3,4,2];
        factors = [options.Resampling_factor, 1, options.Resampling_factor];
    else
        error('Axis not defined')
    end
end
if ~isfield(options,'Rotation_angle')
    Rotation_angle = input('Theta = ');
else
    Rotation_angle=options.Rotation_angle;
end
if ~isfield(options,'Fill_pixel')
    Fill_pixel=avw.hdr.dime.glmin;
else
    Fill_pixel=options.Fill_pixel;
end
if ~isfield(options,'Center_of_rotation') || (length(options.Center_of_rotation)<=3)
    Center_of_rotation = [floor(avw.hdr.dime.dim(dime(1))/2),floor(avw.hdr.dime.dim(dime(2))/2), avw.hdr.dime.dim(dime(3))];
    pCenter_of_rotation = double(Center_of_rotation)./double([avw.hdr.dime.dim(dime(1)),avw.hdr.dime.dim(dime(2)),avw.hdr.dime.dim(dime(3))]);    
else
    Center_of_rotation=options.Center_of_rotation;
    pCenter_of_rotation = Center_of_rotation./double([avw.hdr.dime.dim(dime(1)),avw.hdr.dime.dim(dime(2)),avw.hdr.dime.dim(dime(3))]);
end
%%%%%%%%%% RESAMPLING %%%%%%%%%%%%%%%%%%%%
if isfield(options,'Resampling_factor') && options.Resampling_factor ~=1
    %%%% Avoid mixed values on borders %%%%%
    if  (length(options.Avoid_pixel)~=0) %isfield(options,'Avoid_pixel') ||
        Indexes = [] ;
        for i= 1:length(options.Avoid_pixel)
            Indexes = [Indexes,find(avw.img==options.Avoid_pixel(i))'];
        end
        avw.img(Indexes) = intmin;
         disp('Marking averaged values');        
    else
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('First Resampling');
    avw = resample_avw(avw, factors)
    %clean the borders
    disp('Cleaning borders');
    cb=1/options.Resampling_factor;
    avw.img(1:avw.hdr.dime.dim(dime(1)),1:cb,1:avw.hdr.dime.dim(dime(3)))= Fill_pixel;
    avw.img(1:avw.hdr.dime.dim(dime(1)),avw.hdr.dime.dim(dime(2))-cb+1:avw.hdr.dime.dim(dime(2)),1:avw.hdr.dime.dim(dime(3)))= Fill_pixel;
    avw.img(1:cb,1:avw.hdr.dime.dim(dime(2)),1:avw.hdr.dime.dim(dime(3)))= Fill_pixel;
    avw.img(avw.hdr.dime.dim(dime(1))-cb+1:avw.hdr.dime.dim(2),1:avw.hdr.dime.dim(dime(2)),1:avw.hdr.dime.dim(dime(3)))= Fill_pixel;
    Center_of_rotation = double([avw.hdr.dime.dim(dime(1)),avw.hdr.dime.dim(dime(2)),avw.hdr.dime.dim(dime(3))]).*pCenter_of_rotation;
else
    disp('No Resampling');
end

disp(['Rotating around: [', num2str(Center_of_rotation), '] of ' , num2str(360*Rotation_angle/(2*pi)), ' deg']);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pixdim = [avw.hdr.dime.pixdim(dime(1)),avw.hdr.dime.pixdim(dime(2)),avw.hdr.dime.pixdim(dime(3))];
% Dimension of new box %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BZ = floor(avw.hdr.dime.dim(2)/cos(Rotation_angle));
% HZ = floor(avw.hdr.dime.dim(3)/cos(Rotation_angle));
% pixdim = [pixdim(1)/cos(Rotation_angle),pixdim(2)/cos(Rotation_angle),pixdim(3)];
% mkoptions.dim= dim;
% mkoptions.pixdim =pixdim;
% mkoptions.name =name;
% mkoptions.Fill_pixel =2;
% newavw= blank_avw(mkoptions)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Same dimension of the Original Box
BZ = avw.hdr.dime.dim(dime(1));
HZ = avw.hdr.dime.dim(dime(2));
LZ = avw.hdr.dime.dim(dime(3));
dim = [BZ, HZ, LZ];;
newavw = avw;
%
R = [cos(Rotation_angle) sin(Rotation_angle); -sin(Rotation_angle) cos(Rotation_angle)];
iR = inv(R);
Xg = [double(Center_of_rotation(1));double(Center_of_rotation(2))];
Xg2 = [double(floor(BZ/2));double(floor(HZ/2))];
disp('Image processing');
for i=1:LZ
    for j=1:HZ
        for k=1:BZ
            X2 = [double(k);double(j)];
            X = iR*(X2-Xg2)+Xg;
            x = round(X(1));
            y = round(X(2));
            if x <= 0 || y <=0 || x >= avw.hdr.dime.dim(dime(1)) || y >=avw.hdr.dime.dim(dime(2))
                if options.Axis == 'Z'
                    newavw.img(k,j,i)= Fill_pixel;
                elseif options.Axis == 'X'
                    newavw.img(i,k,j)= Fill_pixel;
                elseif options.Axis == 'Y'
                    newavw.img(j,i,k)= Fill_pixel;
                else
                end
            else
                if options.Axis == 'Z'
                    newavw.img(k,j,i) = avw.img(x,y,i);
                elseif options.Axis == 'X'
                    newavw.img(i,k,j) = avw.img(i,x,y);
                elseif options.Axis == 'Y'
                    newavw.img(j,i,k) = avw.img(y,i,x);
                else
                end
            end
        end
    end
end
disp('Image done');
%%%%%%%%%% RESAMPLING %%%%%%%%%%%%%%%%%%%%
factors = 1./factors;
if isfield(options,'Resampling_factor') && options.Resampling_factor ~=1
    disp('Second Resampling');
    newavw = resample_avw(newavw, factors);
    %%%% Avoid mixed values on borders %%%%%
    if (length(options.Avoid_pixel)~=0) %isfield(options,'Avoid_pixel') || 
        Indexes = [] ;        
            Indexes = [find(newavw.img <= options.Avoid_pixel)'];
        newavw.img(Indexes) = options.Avoid_pixel;
         disp('Eliminating averaged values');
    else
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% avw_img_write(newavw,options.namefile)
% avw_hdr_write(newavw,options.namefile)
end
